""" This module contains utility functions necessary for protocol implementation. """

from base64 import urlsafe_b64decode, urlsafe_b64encode
from struct import pack
from typing import List


# specification: https://tools.ietf.org/html/draft-paragon-paseto-rfc-00#section-2.2.1
def pae(pieces: List[bytes]) -> bytes:
    """Applies Pre-Authentication Encoding (PAE) to input."""

    if not isinstance(pieces, list):
        raise TypeError("Expecting a list of bytes-like objects")

    ret = le64(len(pieces))
    for piece in pieces:
        ret += le64(len(piece)) + piece

    return ret


def le64(num: int) -> bytes:
    """Encodes a 64-bit unsigned integer into a little-endian binary string."""
    return pack("<Q", num)


def b64(input_bytes: bytes) -> bytes:
    """Returns base64 encoding.

    Input is encoded using base64url as defined in RFC4648, without "=" padding.
    """
    # https://tools.ietf.org/html/rfc4648#section-4
    # remove padding
    return urlsafe_b64encode(input_bytes).rstrip(b"=")


def b64decode(input_bytes: bytes) -> bytes:
    """Returns base64 decoding by reversing b64()."""
    return urlsafe_b64decode(input_bytes + b"=" * padding_size(len(input_bytes)))


def padding_size(num: int) -> int:
    """Calculates base64 padding size as per RFC4648."""
    # according to https://tools.ietf.org/html/rfc4648#section-4
    # only three cases can exist
    mapping = {
        0: 0,  # no padding
        2: 2,  # two padding characters
        3: 1,  # one padding character
    }

    lookup = num % 4

    if lookup not in mapping:
        raise ValueError("Impossible input size, unable to calculate padding")

    return mapping[lookup]



""" This module exports third party primitives. """

# from nacl.bindings import crypto_aead_xchacha20poly1305_ietf_decrypt as decrypt
# from nacl.bindings import crypto_aead_xchacha20poly1305_ietf_encrypt as encrypt
from pysodium import crypto_aead_xchacha20poly1305_ietf_decrypt as decrypt
from pysodium import crypto_aead_xchacha20poly1305_ietf_encrypt as encrypt
from pysodium import crypto_sign_detached as sign
from pysodium import crypto_sign_verify_detached as verify

# from libsodium import sign, verify, decrypt, encrypt

__all__ = ["encrypt", "decrypt", "sign", "verify"]



"""This module accesses libsodium via ctypes."""

import ctypes
import ctypes.util

_library = ctypes.util.find_library("sodium") or ctypes.util.find_library("libsodium")
if _library is None:
    raise ValueError("Could not find libsodium")
_sodium = ctypes.cdll.LoadLibrary(_library)


def crypto_stream_xchacha20_xor(message: bytes, nonce: bytes, key: bytes) -> bytes:
    """Gives access to libsodium function of the same name."""

    if len(nonce) != _sodium.crypto_stream_xchacha20_noncebytes():
        raise ValueError("incorrect nonce size")
    if len(key) != _sodium.crypto_stream_xchacha20_keybytes():
        raise ValueError("incorrect key size")

    message_length: ctypes.c_longlong = ctypes.c_longlong(len(message))

    ciphertext = ctypes.create_string_buffer(len(message))

    exit_code = _sodium.crypto_stream_xchacha20_xor(
        ciphertext, message, message_length, nonce, key
    )
    if exit_code != 0:
        raise ValueError

    return ciphertext.raw



""" This module contains exceptions. """


class PasetoException(Exception):
    """Base exception for all paseto related errors."""


class InvalidFooter(PasetoException):
    """Footer could not be verified."""


class InvalidHeader(PasetoException):
    """Message contains incorrect header."""


class InvalidMac(PasetoException):
    """Invalid MAC for given ciphertext in decrypt."""


class InvalidKey(PasetoException):
    """Invalid key for this version of the protocol and method."""



"""This module contains common building blocks used in several protocol versions."""

import hmac

from paseto.exceptions import InvalidFooter, InvalidHeader
from paseto.protocol.util import b64, b64decode


def check_footer(message: bytes, footer: bytes) -> None:
    """Check that message contains a valid footer."""
    if footer and not hmac.compare_digest(b64(footer), message.split(b".")[-1]):
        raise InvalidFooter("Invalid message footer")


def check_header(message: bytes, header: bytes) -> None:
    """Check that message begins with a valid header."""
    if not message.startswith(header):
        raise InvalidHeader("Invalid message header")


def decode_message(message: bytes, header_length: int) -> bytes:
    """Returns message decoded into raw binary."""
    return b64decode(
        # strip header and remove any footer
        message[header_length:].split(b".")[0]
    )



"""
This module contains functions to help manage keys used in PASETO protocols.

It includes an implementation of Algorithm Lucidity.
https://github.com/paseto-standard/paseto-spec/blob/master/docs/02-Implementation-Guide/03-Algorithm-Lucidity.md

A PASERK implementation may supersede this module in the future.
https://github.com/paseto-standard/paserk
"""

import os
from base64 import urlsafe_b64decode, urlsafe_b64encode
from typing import Tuple

import pysodium

_KEY_PREFIX = b"k"
_KEY_LENGHT = 32

_TYPE_LOCAL = b".local."
_TYPE_PUBLIC = b".public."
_TYPE_SECRET = b".secret."


def _create_symmetric_key(version: int, raw_key_material: bytes = b"") -> bytes:
    """Return a new symmetric key."""
    _validate_version(version)
    if not raw_key_material:
        raw_key_material = os.urandom(_KEY_LENGHT)
    return _serialize_key(version, _TYPE_LOCAL, raw_key_material)


def _create_asymmetric_key(
    version: int,
    raw_public_key_material: bytes = b"",
    raw_secret_key_material: bytes = b"",
) -> Tuple[bytes, bytes]:
    """Return new public and secret keys."""
    _validate_version(version)
    if not raw_public_key_material or not raw_secret_key_material:
        (
            raw_public_key_material,
            raw_secret_key_material,
        ) = pysodium.crypto_sign_seed_keypair(
            os.urandom(pysodium.crypto_sign_SEEDBYTES)
        )

    public_key: bytes = _serialize_key(version, _TYPE_PUBLIC, raw_public_key_material)
    secret_key: bytes = _serialize_key(version, _TYPE_SECRET, raw_secret_key_material)
    return public_key, secret_key


def _serialize_key(version: int, key_type: bytes, raw_key: bytes) -> bytes:
    """Returns text representation of raw key bytes."""
    return _get_key_prefix(version, key_type) + urlsafe_b64encode(raw_key)


def _deserialize_key(key: bytes) -> bytes:
    """Returns raw key bytes from a serialised key."""
    return urlsafe_b64decode(key.split(b".")[-1])


def _validate_version(version: int) -> bool:
    """Validate all supported protocol versions."""
    return version in [4]


def _get_key_prefix(version: int, key_type: bytes) -> bytes:
    """Return key prefix for serialization."""
    return _KEY_PREFIX + str(version).encode() + key_type


def _verify_key(key: bytes, version: int, key_type: bytes) -> bool:
    """Verify that key contains correct prefix."""
    return key.startswith(_get_key_prefix(version, key_type))



""" This module contains Version2 implementation of the Paseto protocol. """

import hashlib
import os

from paseto.crypto import primitives
from paseto.protocol.common import check_footer, check_header, decode_message

from .util import b64, pae

HEADER_LOCAL = b"v2.local."
HEADER_PUBLIC = b"v2.public."
NONCE_SIZE = 24


def encrypt(message: bytes, key: bytes, footer: bytes = b"") -> bytes:
    """https://tools.ietf.org/html/draft-paragon-paseto-rfc-00#section-5.3.1"""

    # Given a message "m", key "k", and optional footer "f".

    # 1.  Set header "h" to "v2.local."
    header = HEADER_LOCAL

    # 2.  Generate 24 random bytes from the OS's CSPRNG.
    random_bytes = os.urandom(NONCE_SIZE)

    # 3.  Calculate BLAKE2b of the message "m" with the output of step 2 as
    #        the key, with an output length of 24.  This will be our nonce,
    #        "n".
    #
    #        *  This step is to ensure that an RNG failure does not result in
    #           a nonce-misuse condition that breaks the security of our
    #           stream cipher.
    nonce = get_nonce(message, random_bytes)

    # 4. Pack "h", "n", and "f" together (in that order) using PAE
    pre_auth = pae([header, nonce, footer])

    # 5.  Encrypt the message using XChaCha20-Poly1305, using an AEAD
    #        interface such as the one provided in libsodium.
    cipher_text = primitives.encrypt(message, pre_auth, nonce, key)

    #    6.  If "f" is:
    #
    #        *  Empty: return h || b64(n || c)
    #
    #        *  Non-empty: return h || b64(n || c) || "." || base64url(f)
    #
    #        *  ...where || means "concatenate"
    ret = header + b64(nonce + cipher_text)
    if footer:
        ret += b"." + b64(footer)

    return ret


def decrypt(message: bytes, key: bytes, footer: bytes = b"") -> bytes:
    """https://tools.ietf.org/html/draft-paragon-paseto-rfc-00#section-5.3.2"""

    # Given a message "m", key "k", and optional footer "f".

    #    1.  If "f" is not empty, implementations MAY verify that the value
    #        appended to the token matches some expected string "f", provided
    #        they do so using a constant-time string compare function.
    check_footer(message, footer)

    # 2.  Verify that the message begins with "v2.local.", otherwise throw
    #        an exception.  This constant will be referred to as "h".
    header = HEADER_LOCAL
    check_header(message, header)

    # 3.  Decode the payload ("m" sans "h", "f", and the optional trailing
    #        period between "m" and "f") from base64url to raw binary.  Set:
    #
    #        *  "n" to the leftmost 24 bytes
    #        *  "c" to the middle remainder of the payload, excluding "n".
    raw_inner_message = decode_message(message, len(header))

    nonce = raw_inner_message[:NONCE_SIZE]
    cipher_text = raw_inner_message[NONCE_SIZE:]

    # 4.  Pack "h", "n", and "f" together (in that order) using PAE (see
    #        Section 2.2).  We'll call this "preAuth"
    pre_auth = pae([header, nonce]) if footer is None else pae([header, nonce, footer])

    # 5.  Decrypt "c" using "XChaCha20-Poly1305", store the result in "p".
    # 6.  If decryption failed, throw an exception.  Otherwise, return "p".
    return primitives.decrypt(cipher_text, pre_auth, nonce, key)


def sign(message: bytes, secret_key: bytes, footer: bytes = b"") -> bytes:
    """https://tools.ietf.org/html/draft-paragon-paseto-rfc-00#section-5.3.3"""

    # Given a message "m", Ed25519 secret key "sk", and optional footer "f"
    #    (which defaults to empty string):

    # 1.  Set "h" to "v2.public."
    header = HEADER_PUBLIC

    # 2.  Pack "h", "m", and "f" together (in that order) using PAE. We'll call this "m2".
    message2 = pae([header, message, footer])

    # 3.  Sign "m2" using Ed25519 "sk".  We'll call this "sig".
    signature = primitives.sign(message2, secret_key)

    # 4.  If "f" is:
    #
    #        *  Empty: return h || b64(m || sig)
    #
    #        *  Non-empty: return h || b64(m || sig) || "." || b64(f)
    #
    #        *  ...where || means "concatenate"
    ret = header + b64(message + signature)
    if footer:
        ret += b"." + b64(footer)

    return ret


def verify(signed_message: bytes, public_key: bytes, footer: bytes = b"") -> bytes:
    """https://tools.ietf.org/html/draft-paragon-paseto-rfc-00#section-5.3.4"""

    # Given a signed message "sm", public key "pk", and optional footer "f"
    #    (which defaults to empty string):

    # 1.  If "f" is not empty, implementations MAY verify that the value
    #        appended to the token matches some expected string "f", provided
    #        they do so using a constant-time string compare function.
    check_footer(signed_message, footer)

    # 2.  Verify that the message begins with "v2.public.", otherwise throw
    #        an exception.  This constant will be referred to as "h".
    header = HEADER_PUBLIC
    check_header(signed_message, header)

    # 3.  Decode the payload ("sm" sans "h", "f", and the optional trailing
    #        period between "m" and "f") from base64url to raw binary.  Set:
    #
    #        *  "s" to the rightmost 64 bytes
    #
    #        *  "m" to the leftmost remainder of the payload, excluding "s"
    raw_inner_message = decode_message(signed_message, len(header))

    signature = raw_inner_message[-64:]
    message = raw_inner_message[:-64]

    # 4.  Pack "h", "m", and "f" together (in that order) using PAE. We'll call this "m2".
    message2 = pae([header, message, footer])

    # 5.  Use Ed25519 to verify that the signature is valid for the message
    # 6.  If the signature is valid, return "m".  Otherwise, throw an exception.
    primitives.verify(signature, message2, public_key)
    return message


def get_nonce(message: bytes, random_bytes: bytes) -> bytes:
    """Return nonce per Version2 specification."""
    return hashlib.blake2b(message, key=random_bytes, digest_size=NONCE_SIZE).digest()


# backwards compatibility, do not use this class
# pylint: disable=too-few-public-methods
class Version2:
    """Version2 implementation of the Paseto protocol."""

    HEADER_LOCAL = HEADER_LOCAL
    HEADER_PUBLIC = HEADER_PUBLIC
    NONCE_SIZE = NONCE_SIZE
    encrypt = staticmethod(encrypt)
    decrypt = staticmethod(decrypt)
    sign = staticmethod(sign)
    verify = staticmethod(verify)
    get_nonce = staticmethod(get_nonce)
    check_footer = staticmethod(check_footer)
    check_header = staticmethod(check_header)
    decode_message = staticmethod(decode_message)



# cython: language_level=3

# Cython wrapper for libsodium

# libsodium C API definitions for Cython

cdef extern from "../../submodules/libsodium/src/libsodium/include/sodium/crypto_aead_xchacha20poly1305.h":
    int crypto_aead_xchacha20poly1305_ietf_encrypt(
            unsigned char *c,
            unsigned long long *clen_p,
            const unsigned char *m,
            unsigned long long mlen,
            const unsigned char *ad,
            unsigned long long adlen,
            const unsigned char *nsec,
            const unsigned char *npub,
            const unsigned char *k
    )

    int crypto_aead_xchacha20poly1305_ietf_decrypt(
            unsigned char *m,
            unsigned long long *mlen_p,
            unsigned char *nsec,
            const unsigned char *c,
            unsigned long long clen,
            const unsigned char *ad,
            unsigned long long adlen,
            const unsigned char *npub,
            const unsigned char *k)


cdef extern from "../../submodules/libsodium/src/libsodium/include/sodium/crypto_sign.h":
    int crypto_sign_detached(
            unsigned char *sig,
            unsigned long long *siglen_p,
            const unsigned char *m,
            unsigned long long mlen,
            const unsigned char *sk
    )

    int crypto_sign_verify_detached(
            const unsigned char *sig,
            const unsigned char *m,
            unsigned long long mlen,
            const unsigned char *pk
    )

cdef extern from "../../submodules/libsodium/src/libsodium/include/sodium/crypto_sign.h":
    size_t crypto_sign_bytes()
    size_t crypto_sign_secretkeybytes()
    size_t crypto_sign_publickeybytes()

cdef extern from "../../submodules/libsodium/src/libsodium/include/sodium/crypto_aead_xchacha20poly1305.h":
    size_t crypto_aead_xchacha20poly1305_ietf_abytes()
    size_t crypto_aead_xchacha20poly1305_ietf_npubbytes()
    size_t crypto_aead_xchacha20poly1305_ietf_keybytes()

NONCE_SIZE = 'Invalid nonce length'
KEY_SIZE = 'Invalid key length'
PUBLIC_KEY_SIZE = 'Invalid public key length'
SECRET_KEY_SIZE = 'Invalid secret key length'
SIGNATURE_SIZE = 'Invalid signature length'

# https://libsodium.gitbook.io/doc/secret-key_cryptography/aead/chacha20-poly1305/xchacha20-poly1305_construction
def encrypt(message: bytes, ad: bytes, nonce: bytes, key: bytes):

    check_nonce_size(len(nonce))
    check_key_size(len(key))

    cipher_text = b' ' * (len(message) + crypto_aead_xchacha20poly1305_ietf_abytes())

    cdef unsigned long long ciphertext_length = len(cipher_text)

    crypto_aead_xchacha20poly1305_ietf_encrypt(cipher_text, &ciphertext_length, message, len(message), ad, len(ad), NULL, nonce, key)

    return cipher_text

def decrypt(cipher_text: bytes, aad: bytes, nonce: bytes, key: bytes):

    check_nonce_size(len(nonce))
    check_key_size(len(key))

    cdef size_t cipher_text_length = len(cipher_text)

    message = b' ' * (cipher_text_length - crypto_aead_xchacha20poly1305_ietf_abytes())
    cdef unsigned long long message_length

    res = crypto_aead_xchacha20poly1305_ietf_decrypt(message, &message_length, NULL, cipher_text, cipher_text_length, aad, len(aad), nonce, key)

    if res != 0:
        raise Exception('Decryption failed')

    return message

def sign(message: bytes, secret_key: bytes):

    check_secret_key_size(len(secret_key))

    signature = b' ' * crypto_sign_bytes()
    cdef unsigned long long signature_length = len(signature)

    crypto_sign_detached(signature, &signature_length, message, len(message), secret_key)

    return signature

def verify(signature: bytes, message: bytes, public_key: bytes):

    check_signature_size(len(signature))
    check_public_key_size(len(public_key))

    crypto_sign_verify_detached(signature, message, len(message), public_key)

def check_size(actual_size, expected_size, exception_message):
    """ Helper method to verify length of method call arguments """
    if actual_size != expected_size:
        raise ValueError(exception_message)

def check_key_size(key_length: int):
    check_size(key_length, crypto_aead_xchacha20poly1305_ietf_keybytes(), KEY_SIZE)

def check_nonce_size(nonce_length: int):
    check_size(nonce_length, crypto_aead_xchacha20poly1305_ietf_npubbytes(), NONCE_SIZE)

def check_public_key_size(public_key_length: int):
    check_size(public_key_length, crypto_sign_publickeybytes(), PUBLIC_KEY_SIZE)

def check_secret_key_size(secret_key_length: int):
    check_size(secret_key_length, crypto_sign_secretkeybytes(), SECRET_KEY_SIZE)

def check_signature_size(signature_length: int):
    check_size(signature_length, crypto_sign_bytes(), SIGNATURE_SIZE)

__all__ = ['encrypt', 'decrypt', 'sign', 'verify']



"""
This module contains Version4 implementation of the Paseto protocol.

https://github.com/paseto-standard/paseto-spec/blob/master/docs/01-Protocol-Versions/Version4.md
"""

import hashlib
import hmac
import os
from typing import Tuple

from paseto.crypto import libsodium_wrapper, primitives
from paseto.exceptions import InvalidKey, InvalidMac
from paseto.paserk.keys import (
    _TYPE_LOCAL,
    _TYPE_PUBLIC,
    _TYPE_SECRET,
    _create_asymmetric_key,
    _create_symmetric_key,
    _deserialize_key,
)
from paseto.paserk.keys import _verify_key as _generic_verify_key
from paseto.protocol.common import check_footer, check_header, decode_message
from paseto.protocol.util import b64, pae

HEADER_LOCAL = b"v4.local."
HEADER_PUBLIC = b"v4.public."

NONCE_SIZE = 32
ENCRYPTION_KEY_LENGTH = 32
AUTHENTICATION_KEY_LENGTH = 32
MAC_SIZE = 32

INFO_ENCRYPTION = b"paseto-encryption-key"
INFO_AUTHENTICATION = b"paseto-auth-key-for-aead"


def encrypt(
    message: bytes, key: bytes, footer: bytes = b"", implicit_assertion: bytes = b""
) -> bytes:
    """PASETO Version4 encrypt function."""

    # verify that key is intended for use with this function
    _verify_key(key, _TYPE_LOCAL)
    raw_key: bytes = _deserialize_key(key)

    # Step 1
    header: bytes = HEADER_LOCAL

    # Step 2
    nonce: bytes = os.urandom(NONCE_SIZE)

    # Step 3
    encryption_key: bytes
    authentication_key: bytes
    nonce2: bytes
    encryption_key, authentication_key, nonce2 = _split_key(raw_key, nonce)

    # Step 4
    ciphertext: bytes = libsodium_wrapper.crypto_stream_xchacha20_xor(
        message=message, nonce=nonce2, key=encryption_key
    )

    # Step 5
    pre_auth: bytes = pae([header, nonce, ciphertext, footer, implicit_assertion])

    # Step 6
    message_authentication_code: bytes = hashlib.blake2b(
        pre_auth, key=authentication_key, digest_size=32
    ).digest()

    # Step 7
    ret: bytes = header + b64(nonce + ciphertext + message_authentication_code)
    if footer:
        ret += b"." + b64(footer)
    return ret


def decrypt(
    message: bytes, key: bytes, footer: bytes = b"", implicit_assertion: bytes = b""
) -> bytes:
    """PASETO Version4 decrypt function."""

    # verify that key is intended for use with this function
    _verify_key(key, _TYPE_LOCAL)
    raw_key: bytes = _deserialize_key(key)

    # Step 1
    check_footer(message, footer)

    # Step 2
    header: bytes = HEADER_LOCAL
    check_header(message, header)

    # Step 3
    decoded: bytes = decode_message(message, len(header))
    nonce: bytes = decoded[:NONCE_SIZE]
    ciphertext: bytes = decoded[NONCE_SIZE:-MAC_SIZE]

    # Step 4
    encryption_key, authentication_key, nonce2 = _split_key(raw_key, nonce)

    # Step 5
    pre_auth: bytes = pae([header, nonce, ciphertext, footer, implicit_assertion])

    # Step 6
    computed_mac: bytes = hashlib.blake2b(
        pre_auth, key=authentication_key, digest_size=MAC_SIZE
    ).digest()

    # Step 7
    mac_in_message: bytes = decoded[-MAC_SIZE:]
    if not hmac.compare_digest(mac_in_message, computed_mac):
        raise InvalidMac("Invalid MAC for given ciphertext")

    # Steps 8 and 9
    return libsodium_wrapper.crypto_stream_xchacha20_xor(
        message=ciphertext, nonce=nonce2, key=encryption_key
    )


def sign(
    message: bytes,
    secret_key: bytes,
    footer: bytes = b"",
    implicit_assertion: bytes = b"",
) -> bytes:
    """Sign message and return token which can then be used with verify()."""

    # verify that key is intended for use with this function
    _verify_key(secret_key, _TYPE_SECRET)
    raw_secret_key: bytes = _deserialize_key(secret_key)

    # Step 1
    header = HEADER_PUBLIC

    # Step 2
    message2 = pae([header, message, footer, implicit_assertion])

    # Step 3
    signature = primitives.sign(message2, raw_secret_key)

    # Step 4
    ret = header + b64(message + signature)
    if footer:
        ret += b"." + b64(footer)

    return ret


def verify(
    signed_message: bytes,
    public_key: bytes,
    footer: bytes = b"",
    implicit_assertion: bytes = b"",
) -> bytes:
    """Verify signature and return message. Raises exception if signature is invalid."""

    # verify that key is intended for use with this function
    _verify_key(public_key, _TYPE_PUBLIC)
    raw_public_key: bytes = _deserialize_key(public_key)

    # Step 1
    check_footer(signed_message, footer)

    # Step 2
    header = HEADER_PUBLIC
    check_header(signed_message, header)

    # Step 3
    raw_inner_message: bytes = decode_message(signed_message, len(header))
    signature = raw_inner_message[-64:]
    message = raw_inner_message[:-64]

    # Step 4
    message2 = pae([header, message, footer, implicit_assertion])

    # Steps 5 and 6
    primitives.verify(signature, message2, raw_public_key)
    return message


def _split_key(key: bytes, nonce: bytes) -> Tuple[bytes, bytes, bytes]:

    hashed: bytes = hashlib.blake2b(
        INFO_ENCRYPTION + nonce, key=key, digest_size=56
    ).digest()
    encryption_key: bytes = hashed[:ENCRYPTION_KEY_LENGTH]
    nonce2: bytes = hashed[ENCRYPTION_KEY_LENGTH:]
    authentication_key: bytes = hashlib.blake2b(
        INFO_AUTHENTICATION + nonce, key=key, digest_size=AUTHENTICATION_KEY_LENGTH
    ).digest()

    return encryption_key, authentication_key, nonce2


def _verify_key(key: bytes, key_type: bytes) -> None:
    if not _generic_verify_key(key, 4, key_type):
        raise InvalidKey


def create_symmetric_key() -> bytes:
    """Return key for use with encrypt() and decrypt()."""
    return _create_symmetric_key(4)


def create_asymmetric_key() -> Tuple[bytes, bytes]:
    """Return key pair for use with sign() and verify()."""
    return _create_asymmetric_key(4)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['paseto', 'paseto.crypto', 'paseto.paserk', 'paseto.protocol']

package_data = \
{'': ['*']}

install_requires = \
['pysodium>=0.7.9,<0.8.0']

setup_kwargs = {
    'name': 'python-paseto',
    'version': '0.5.0',
    'description': 'Platform-Agnostic Security Tokens for Python',
    'long_description': '# python-paseto\nPlatform-Agnostic Security Tokens for Python\n\n[![Build Status](https://travis-ci.com/purificant/python-paseto.svg?branch=main)](https://travis-ci.com/purificant/python-paseto)\n[![test-workflow](https://github.com/purificant/python-paseto/actions/workflows/test.yaml/badge.svg)](https://github.com/purificant/python-paseto/actions/workflows/test.yaml)\n[![PyPI version](https://badge.fury.io/py/python-paseto.svg)](https://badge.fury.io/py/python-paseto)\n[![Coverage Status](https://coveralls.io/repos/github/purificant/python-paseto/badge.svg?branch=main)](https://coveralls.io/github/purificant/python-paseto?branch=main)\n[![Language grade: Python](https://img.shields.io/lgtm/grade/python/g/purificant/python-paseto.svg?logo=lgtm&logoWidth=18)](https://lgtm.com/projects/g/purificant/python-paseto/context:python)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n\n\n# Installation\n\n```bash\npip install python-paseto\n```\n\n### Check installation\n```python\npython -m paseto\n```\n`libsodium` is required, this will check if it is installed on your system. On Ubuntu 20.04 you can get it with `sudo apt install libsodium23`.\n\n# Low level API\nImplements PASETO Version2 and Version4 protocols supporting `v2.public`, `v2.local`, `v4.public` and `v4.local` messages.\nEvery protocol version provides access to encrypt() / decrypt() and sign() / verify() functions.\n\nLow level API is focuses on solid, high quality, production ready primitives\nas specified directly in the [PASETO](https://tools.ietf.org/html/draft-paragon-paseto-rfc-00) \nprotocol.\nSee [paseto-spec](https://github.com/paseto-standard/paseto-spec) for protocol details.\n\n# Example use with Version2\n```python\nfrom paseto.protocol.version2 import encrypt, decrypt\n\nmessage = b"foo"  # your data\nkey = b"0" * 32  # encryption key\n\ntoken = encrypt(message, key)\nplain_text = decrypt(token, key)\n\nassert plain_text == message\nprint(f"token={token}")\nprint(f"plain_text={plain_text}")\nprint(f"message={message}")\n```\n### With optional footer\n```python\nfrom paseto.protocol.version2 import encrypt, decrypt\n\nmessage = b"foo"  # your data\nkey = b"0" * 32  # encryption key\noptional_footer = b"sample_footer"  # authenticated but not encrypted metadata\n\ntoken = encrypt(message, key, optional_footer)\nplain_text = decrypt(token, key, optional_footer)\n\nassert plain_text == message\nprint(f"token={token}")\nprint(f"plain_text={plain_text}")\nprint(f"message={message}")\n```\n\n# Example use with Version4\n```python\nfrom paseto.protocol.version4 import create_symmetric_key, decrypt, encrypt\n\nmessage = b"this is a secret message"  # your data\nkey = create_symmetric_key()  # encryption key\n\ntoken = encrypt(message, key)\nplain_text = decrypt(token, key)\n\nassert plain_text == message\nprint(f"token={token}")\nprint(f"plain_text={plain_text}")\nprint(f"message={message}")\n```\n\n### Message signing\n```python\nfrom paseto.protocol.version4 import create_asymmetric_key, sign, verify\n\nmessage = b"this is a public message"  # your data\npublic_key, secret_key = create_asymmetric_key()  # signing / verifying keys\n\ntoken = sign(message, secret_key)\nverified_message = verify(token, public_key)\n\nassert verified_message == message\nprint(f"token={token}")\nprint(f"verified_message={verified_message}")\nprint(f"message={message}")\n```\n\n# High level API\nIn the future a high level API will provide developer friendly access to low level API\nand support easy integration into other projects.\n\n# Development\nTypical dev workflow operations are automated in [Makefile](https://github.com/purificant/python-paseto/blob/main/Makefile),\nincluding testing, linting, code quality checks, benchmarks and dev environment setup.\n\n# Contributing\nThis library is under active development and maintenance. For any feedback, questions,\ncomments or if you would like to request a feature, please raise an issue!\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/purificant/python-paseto',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<3.10',
}


setup(**setup_kwargs)
